package fr.inria.eventloud.logs_analyzer;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;

import org.jdesktop.swingx.JXTable;
import org.jdesktop.swingx.decorator.HighlighterFactory;

/**
 * 
 * @author lpellegr
 */
public class LogsAnalyzerGUI {

    private JFrame frmLogsanalyzer;

    private final Action quitAction = new QuitAction();

    /**
     * Create the application.
     */
    public LogsAnalyzerGUI(List<SocialFilterResult> entries) {
        this.initialize(entries);
        this.frmLogsanalyzer.setVisible(true);
        this.frmLogsanalyzer.pack();
        this.frmLogsanalyzer.setMinimumSize(this.frmLogsanalyzer.getSize());
        this.frmLogsanalyzer.setLocationRelativeTo(null);
    }

    /**
     * Initialize the contents of the frame.
     * 
     * @param entries
     */
    private void initialize(List<SocialFilterResult> entries) {
        this.frmLogsanalyzer = new JFrame();
        this.frmLogsanalyzer.setTitle("Social Filter Logs Analyzer");
        this.frmLogsanalyzer.setBounds(100, 100, 590, 330);
        this.frmLogsanalyzer.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        JMenuBar menuBar = new JMenuBar();
        this.frmLogsanalyzer.getContentPane().add(menuBar, BorderLayout.NORTH);

        JMenu mnFile = new JMenu("File");
        menuBar.add(mnFile);

        JMenuItem mntmQuit = new JMenuItem("Quit");
        mntmQuit.setAction(this.quitAction);
        mnFile.add(mntmQuit);

        JXTable jxTable = this.initTable(entries);
        this.configureJXTable(jxTable);

        // Create the scroll pane and add the table to it.
        JScrollPane scrollPane = new JScrollPane(jxTable);
        JComponent content = new JPanel(new BorderLayout());

        // Add the scroll pane to this panel.
        content.add(scrollPane, BorderLayout.CENTER);

        this.frmLogsanalyzer.getContentPane().add(content, BorderLayout.CENTER);
    }

    /**
     * Initialize our JXTable; this is standard stuff, just as with JTable
     * 
     * @param entries
     */
    private JXTable initTable(List<SocialFilterResult> entries) {
        JXTable table = new JXTable();
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table.setRowHeight(23);
        table.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    JTable jtable = (JTable) e.getSource();
                    int rowIndex = jtable.getSelectedRow();

                    SocialFilterResult entry =
                            ((SocialFilterResultsTableModel) jtable.getModel()).getEntry(rowIndex);

                    JOptionPane.showMessageDialog(
                            frmLogsanalyzer,
                            "Graph=" + entry.getGraph() + "\nSubject="
                                    + entry.getSubject() + "\nPredicate="
                                    + entry.getPredicate() + "\nObject="
                                    + entry.getObject(),
                            "Last quadruple triggering the social filter verification",
                            JOptionPane.INFORMATION_MESSAGE);
                }
            }
        });

        SocialFilterResultsTableModel model =
                new SocialFilterResultsTableModel(entries);
        table.setModel(model);
        model.loadData();
        return table;
    }

    /**
     * For demo purposes, the special features of the JXTable are configured
     * here. There is otherwise no reason not to do this in initTable().
     */
    private void configureJXTable(JXTable jxTable) {
        // This shows the column control on the right-hand of the header.
        // All there is to it--users can now select which columns to view
        jxTable.setColumnControlVisible(true);

        // We'll add a highlighter to offset different row numbers
        // Note the setHighlighters() takes an array parameter; you can chain
        // these together.
        jxTable.setHighlighters(HighlighterFactory.createSimpleStriping());

        // Renderer for date type column
        jxTable.getColumnModel().getColumn(1).setCellRenderer(
                new DateCellRenderer());

        jxTable.setEditable(false);
        jxTable.packTable(10);
    }

    private class SocialFilterResultsTableModel extends DefaultTableModel {
        private static final long serialVersionUID = 1L;

        private final List<SocialFilterResult> entries;

        public SocialFilterResultsTableModel(List<SocialFilterResult> entries) {
            this.entries = entries;
        }

        public void loadData() {
            this.addColumn("");
            this.addColumn("Date");
            this.addColumn("Source");
            this.addColumn("Destination");
            this.addColumn("Threshold");
            this.addColumn("Relationship Strengh");

            for (SocialFilterResult entry : entries) {
                String iconName = "accept";
                if (entry.getRelationshipStrengh() < entry.getThreshold()) {
                    iconName = "exclamation";
                }

                this.addRow(new Object[] {
                        new ImageIcon(this.getClass().getResource(
                                "/" + iconName + ".png")),
                        entry.getTimeOfOccurence(), entry.getSource(),
                        entry.getDestination(), entry.getThreshold(),
                        entry.getRelationshipStrengh()});
            }
        }

        public SocialFilterResult getEntry(int rowIndex) {
            return this.entries.get(rowIndex);
        }

        @Override
        public Class<?> getColumnClass(int columnIndex) {
            switch (columnIndex) {
                case 0:
                    return Icon.class;
                case 1:
                    return Date.class;
                case 4:
                case 5:
                    return Double.class;
                default:
                    return super.getColumnClass(columnIndex);
            }
        }

    }

    private class DateCellRenderer extends DefaultTableCellRenderer {
        private static final long serialVersionUID = 1L;

        public Component getTableCellRendererComponent(JTable table,
                                                       Object value,
                                                       boolean isSelected,
                                                       boolean hasFocus,
                                                       int row, int column) {
            super.getTableCellRendererComponent(
                    table, value, isSelected, hasFocus, row, column);
            if (value instanceof Date) {
                String strDate =
                        new SimpleDateFormat("dd/MM/yyyy HH:mm:ss.SSS").format((Date) value);
                this.setText(strDate);
            }

            return this;
        }
    }

    private class QuitAction extends AbstractAction {
        private static final long serialVersionUID = 1L;

        public QuitAction() {
            this.putValue(NAME, "Quit");
            this.putValue(SHORT_DESCRIPTION, "Quit the application");
        }

        public void actionPerformed(ActionEvent e) {
            System.exit(0);
        }
    }

}
