package fr.inria.eventloud.logs_analyzer;

import java.awt.EventQueue;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 
 * @author lpellegr
 */
public class Main {

    private static final Logger log = LoggerFactory.getLogger(Main.class);

    public static void main(String[] args) {
        if (args.length < 1) {
            printUsage();
        }

        List<File> filesToParse = new ArrayList<File>();

        File file;
        for (String arg : args) {
            file = new File(arg);

            if (!file.exists()) {
                exit("File '" + file + "' does not exist");
            }

            if (file.isDirectory()) {
                for (File f : file.listFiles()) {
                    if (!f.isDirectory()) {
                        filesToParse.add(f);
                    }
                }
            } else {
                filesToParse.add(file);
            }
        }

        log.info("Parsing {} file(s)", filesToParse.size());

        SocialFilterParser parser = new SocialFilterParser();
        final List<SocialFilterResult> results = parser.parse(filesToParse);

        if (results.size() > 0) {
            log.info("Creating GUI for analyzing information");

            EventQueue.invokeLater(new Runnable() {
                public void run() {
                    try {
                        new LogsAnalyzerGUI(results);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            });
        } else {
            log.info("No social filter information found in the specified files");
        }
    }

    private static void exit(String msg) {
        System.err.println(msg);
        System.exit(1);
    }

    private static void printUsage() {
        System.out.println("Usage: java " + Main.class.getName() + " FILES...");
        System.exit(0);
    }

}
