package fr.inria.eventloud.logs_analyzer;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 
 * @author lpellegr
 */
public class SocialFilterParser implements Parser<SocialFilterResult> {

    private static final DateFormat DATE_FORMATTER = new SimpleDateFormat(
            "yyyy.MM.dd HH.mm.ss.SSS");

    private static final Pattern PATTERN =
            Pattern.compile("(\\d{4})-(\\d{2})-(\\d{2}) (\\d{2}):(\\d{2}):(\\d{2}).(\\d{3}) \\[DEBUG  PublishSubscribeUtils\\] SocialFilterAnswer\\[source=([^, ]*), destination=([^, ]*), threshold=([^, ]*), relationship_strengh=([^, ]*), quadruple=([^, ]*)\\]");

    public List<SocialFilterResult> parse(List<File> files) {
        List<SocialFilterResult> result = new ArrayList<SocialFilterResult>();

        for (File file : files) {
            result.addAll(this.parse(file));
        }

        return result;
    }

    public List<SocialFilterResult> parse(File file) {
        List<SocialFilterResult> result = new ArrayList<SocialFilterResult>();
        try {
            BufferedReader br = new BufferedReader(new FileReader(file));

            String line;
            while ((line = br.readLine()) != null) {
                Matcher matcher = PATTERN.matcher(line);

                if (matcher.matches()) {
                    String yyyy = matcher.group(1);
                    String MM = matcher.group(2);
                    String dd = matcher.group(3);
                    String HH = matcher.group(4);
                    String mm = matcher.group(5);
                    String ss = matcher.group(6);
                    String SSS = matcher.group(7);

                    String dateString =
                            yyyy + '.' + MM + '.' + dd + ' ' + HH + '.' + mm
                                    + '.' + ss + "." + SSS;

                    Date timeOfOccurence = null;
                    try {
                        timeOfOccurence = DATE_FORMATTER.parse(dateString);
                    } catch (ParseException e) {
                        e.printStackTrace();
                    }

                    String source = matcher.group(8);
                    String destination = matcher.group(9);
                    double threshold = Double.parseDouble(matcher.group(10));
                    double relationshipStrengh =
                            Double.parseDouble(matcher.group(11));

                    String[] terms =
                            matcher.group(12).split(Pattern.quote("|"));
                    String graph = terms[0];
                    String subject = terms[1];
                    String predicate = terms[2];
                    String object = terms[3];

                    result.add(new SocialFilterResult(
                            timeOfOccurence, source, destination, threshold,
                            relationshipStrengh, graph, subject, predicate,
                            object));

                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }

        return result;
    }

}
